@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,19 +24,23 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
+#ifdef GDK_WINDOWING_X11
+#  include <gdk/gdkx.h>
+#endif /* GDK_WINDOWING_X11 */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+#if HAVE_SYS_WAIT_H
+#  include <sys/wait.h>
+#endif
 #include <signal.h>
 #include <unistd.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "manage_window.h"
@@ -47,7 +51,6 @@
 #include "action.h"
 #include "compose.h"
 #include "procmsg.h"
-#include "gtkstext.h"
 #include "textview.h"
 
 typedef struct _Children		Children;
@@ -72,7 +75,8 @@ struct _Children
 	gboolean	 output;
 
 	GtkWidget	*msg_text;
-	GdkFont		*msgfont;
+
+	gboolean	 is_selection;
 };
 
 struct _ChildInfo
@@ -113,7 +117,6 @@ static void message_actions_execute	(MessageView	*msgview,
 static gboolean execute_actions		(gchar		*action, 
 					 GSList		*msg_list, 
 					 GtkWidget	*text,
-					 GdkFont	*msgfont,
 					 gint		 body_pos,
 					 MimeInfo	*partinfo);
 
@@ -471,7 +474,7 @@ static void compose_actions_execute_cb(Compose *compose, guint action_nb,
 		return;
 	}
 
-	execute_actions(action, NULL, compose->text, NULL, 0, NULL);
+	execute_actions(action, NULL, compose->text, 0, NULL);
 }
 
 static void mainwin_actions_execute_cb(MainWindow *mainwin, guint action_nb,
@@ -503,7 +506,6 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	gchar *buf;
 	gchar *action;
 	GtkWidget *text = NULL;
-	GdkFont *msgfont = NULL;
 	guint body_pos = 0;
 
 	g_return_if_fail(action_nb < g_slist_length(prefs_common.actions_list));
@@ -511,7 +513,7 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	buf = (gchar *)g_slist_nth_data(prefs_common.actions_list, action_nb);
 
 	g_return_if_fail(buf);
-	g_return_if_fail(action = strstr(buf, ": "));
+	g_return_if_fail((action = strstr(buf, ": ")));
 
 	/* Point to the beginning of the command-line */
 	action += 2;
@@ -519,17 +521,16 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	textview = messageview_get_current_textview(msgview);
 	if (textview) {
 		text     = textview->text;
-		msgfont  = textview->msgfont;
 		body_pos = textview->body_pos;
 	}
 	partinfo = messageview_get_selected_mime_part(msgview);
 
-	execute_actions(action, msg_list, text, msgfont, body_pos, partinfo);
+	execute_actions(action, msg_list, text, body_pos, partinfo);
 }
 
 static gboolean execute_actions(gchar *action, GSList *msg_list,
-				GtkWidget *text, GdkFont *msgfont,
-				gint body_pos, MimeInfo *partinfo)
+				GtkWidget *text, gint body_pos,
+				MimeInfo *partinfo)
 {
 	GSList *children_list = NULL;
 	gint is_ok  = TRUE;
@@ -539,11 +540,12 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 	ActionType action_type;
 	MsgInfo *msginfo;
 	gchar *cmd;
-	guint start = 0, end = 0;
 	gchar *sel_str = NULL;
 	gchar *msg_str = NULL;
 	gchar *user_str = NULL;
 	gchar *user_hidden_str = NULL;
+	GtkTextIter start_iter, end_iter;
+	gboolean is_selection = FALSE;
 
 	g_return_val_if_fail(action && *action, FALSE);
 
@@ -570,32 +572,20 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 	}
 
 	if (text) {
-		if (GTK_EDITABLE(text)->has_selection) {
-			start = GTK_EDITABLE(text)->selection_start_pos;
-			end   = GTK_EDITABLE(text)->selection_end_pos;
-			if (start > end) {
-				guint tmp;
-				tmp = start;
-				start = end;
-				end = tmp;
-			}
+		GtkTextBuffer *textbuf;
 
-			if (start == end) {
-				start = body_pos;
-				end = gtk_stext_get_length(GTK_STEXT(text));
-				msg_str = gtk_editable_get_chars
-					(GTK_EDITABLE(text), start, end);
-			} else {
-				sel_str = gtk_editable_get_chars
-					(GTK_EDITABLE(text), start, end);
-				msg_str = g_strdup(sel_str);
-			}
-		} else {
-			start = body_pos;
-			end = gtk_stext_get_length(GTK_STEXT(text));
-			msg_str = gtk_editable_get_chars(GTK_EDITABLE(text),
-							 start, end);
+		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+		is_selection = gtk_text_buffer_get_selection_bounds
+			(textbuf, &start_iter, &end_iter);
+		if (!is_selection) {
+			gtk_text_buffer_get_iter_at_offset
+				(textbuf, &start_iter, body_pos);
+			gtk_text_buffer_get_end_iter(textbuf, &end_iter);
 		}
+		msg_str = gtk_text_buffer_get_text
+			(textbuf, &start_iter, &end_iter, FALSE);
+		if (is_selection)
+			sel_str = g_strdup(msg_str);
 	}
 
 	if (action_type & ACTION_USER_STR) {
@@ -616,19 +606,18 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 		}
 	}
 
-	if (action_type & ACTION_PIPE_OUT) {
-		gtk_stext_freeze(GTK_STEXT(text));
-		gtk_stext_set_point(GTK_STEXT(text), start);
-		gtk_stext_forward_delete(GTK_STEXT(text), end - start);
-		gtk_stext_thaw(GTK_STEXT(text));
+	if (text && (action_type & ACTION_PIPE_OUT)) {
+		GtkTextBuffer *textbuf;
+		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+		gtk_text_buffer_delete(textbuf, &start_iter, &end_iter);
 	}
 
 	children = g_new0(Children, 1);
 
-	children->action      = g_strdup(action);
-	children->action_type = action_type;
-	children->msg_text    = text;
-	children->msgfont     = msgfont;
+	children->action       = g_strdup(action);
+	children->action_type  = action_type;
+	children->msg_text     = text;
+	children->is_selection = is_selection;
 
 	if ((action_type & (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)) &&
 	    ((action_type & ACTION_SINGLE) == 0 || msg_list_len == 1))
@@ -701,6 +690,7 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			     Children *children)
 {
+#ifdef G_OS_UNIX
 	gint chld_in[2], chld_out[2], chld_err[2], chld_status[2];
 	gchar *cmdline[4];
 	pid_t pid, gch_pid;
@@ -739,7 +729,9 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		if (setpgid(0, 0))
 			perror("setpgid");
 
+#ifdef GDK_WINDOWING_X11
 		close(ConnectionNumber(gdk_display));
+#endif /* GDK_WINDOWING_X11 */
 
 		gch_pid = fork();
 
@@ -856,10 +848,14 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 	}
 
 	return child_info;
+#else
+	return NULL;
+#endif /* G_OS_UNIX */
 }
 
 static void kill_children_cb(GtkWidget *widget, gpointer data)
 {
+#ifdef G_OS_UNIX
 	GSList *cur;
 	Children *children = (Children *) data;
 	ChildInfo *child_info;
@@ -870,6 +866,7 @@ static void kill_children_cb(GtkWidget *widget, gpointer data)
 		if (child_info->pid && kill(-child_info->pid, SIGTERM) < 0)
 			perror("kill");
 	}
+#endif /* G_OS_UNIX */
 }
 
 static gint wait_for_children(Children *children)
@@ -931,8 +928,10 @@ static void hide_io_dialog_cb(GtkWidget *w, gpointer data)
 	Children *children = (Children *)data;
 
 	if (!children->nb) {
-		gtk_signal_disconnect_by_data(GTK_OBJECT(children->dialog),
-					      children);
+		g_signal_handlers_disconnect_matched
+			(G_OBJECT(children->dialog),
+			 (GSignalMatchType)G_SIGNAL_MATCH_DATA,
+			 0, 0, NULL, NULL, children);
 		gtk_widget_destroy(children->dialog);
 		free_children(children);
 	}
@@ -999,22 +998,25 @@ static void update_io_dialog(Children *children)
 		if (children->input_hbox)
 			gtk_widget_set_sensitive(children->input_hbox, FALSE);
 		gtk_widget_grab_focus(children->close_btn);
-		gtk_signal_connect(GTK_OBJECT(children->dialog),
-				   "key_press_event",
-				   GTK_SIGNAL_FUNC(io_dialog_key_pressed_cb),
-				   children);
+		g_signal_connect(G_OBJECT(children->dialog),
+				 "key_press_event",
+				 G_CALLBACK(io_dialog_key_pressed_cb),
+				 children);
 	}
 
 	if (children->output) {
 		GtkWidget *text = children->text;
+		GtkTextBuffer *textbuf;
+		GtkTextIter iter, start_iter, end_iter;
 		gchar *caption;
 		ChildInfo *child_info;
 
 		gtk_widget_show(children->scrolledwin);
-		gtk_text_freeze(GTK_TEXT(text));
-		gtk_text_set_point(GTK_TEXT(text), 0);
-		gtk_text_forward_delete(GTK_TEXT(text), 
-					gtk_text_get_length(GTK_TEXT(text)));
+		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+		gtk_text_buffer_get_bounds(textbuf, &start_iter, &end_iter);
+		gtk_text_buffer_delete(textbuf, &start_iter, &end_iter);
+		gtk_text_buffer_get_start_iter(textbuf, &iter);
+
 		for (cur = children->list; cur; cur = cur->next) {
 			child_info = (ChildInfo *)cur->data;
 			if (child_info->pid)
@@ -1026,14 +1028,12 @@ static void update_io_dialog(Children *children)
 					(_("--- Ended: %s\n"),
 					 child_info->cmd);
 
-			gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-					caption, -1);
-			gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-					child_info->output->str, -1);
+			gtk_text_buffer_insert(textbuf, &iter, caption, -1);
+			gtk_text_buffer_insert(textbuf, &iter,
+					       child_info->output->str, -1);
 			g_free(caption);
 			child_info->new_out = FALSE;
 		}
-		gtk_text_thaw(GTK_TEXT(text));
 	}
 }
 
@@ -1060,11 +1060,11 @@ static void create_io_dialog(Children *children)
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Action's input/output"));
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 	manage_window_set_transient(GTK_WINDOW(dialog));
-	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-			GTK_SIGNAL_FUNC(delete_io_dialog_cb), children);
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-			GTK_SIGNAL_FUNC(hide_io_dialog_cb),
-			children);
+	g_signal_connect(G_OBJECT(dialog), "delete_event",
+			 G_CALLBACK(delete_io_dialog_cb), children);
+	g_signal_connect(G_OBJECT(dialog), "destroy",
+			 G_CALLBACK(hide_io_dialog_cb),
+			 children);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
@@ -1077,16 +1077,30 @@ static void create_io_dialog(Children *children)
 
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
-				       GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
+					    GTK_SHADOW_IN);
 	gtk_box_pack_start(GTK_BOX(vbox), scrolledwin, TRUE, TRUE, 0);
-	gtk_widget_set_usize(scrolledwin, 480, 200);
+	gtk_widget_set_size_request(scrolledwin, 560, 200);
 	gtk_widget_hide(scrolledwin);
 
-	text = gtk_text_new(gtk_scrolled_window_get_hadjustment
-			    (GTK_SCROLLED_WINDOW(scrolledwin)),
-			    gtk_scrolled_window_get_vadjustment
-			    (GTK_SCROLLED_WINDOW(scrolledwin)));
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
+	text = gtk_text_view_new();
+
+	if (prefs_common.textfont) {
+		PangoFontDescription *font_desc;
+		font_desc = pango_font_description_from_string
+			(prefs_common.textfont);
+		if (font_desc) {
+			gtk_widget_modify_font(text, font_desc);
+			pango_font_description_free(font_desc);
+		}
+	}
+
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
+	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text), 6);
+	gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text), 6);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), text);
 	gtk_widget_show(text);
 
@@ -1095,17 +1109,17 @@ static void create_io_dialog(Children *children)
 		gtk_widget_show(input_hbox);
 
 		entry = gtk_entry_new();
-		gtk_widget_set_usize(entry, 320, -1);
-		gtk_signal_connect(GTK_OBJECT(entry), "activate",
-				   GTK_SIGNAL_FUNC(send_input), children);
+		gtk_widget_set_size_request(entry, 320, -1);
+		g_signal_connect(G_OBJECT(entry), "activate",
+				 G_CALLBACK(send_input), children);
 		gtk_box_pack_start(GTK_BOX(input_hbox), entry, TRUE, TRUE, 0);
 		if (children->action_type & ACTION_USER_HIDDEN_IN)
 			gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 		gtk_widget_show(entry);
 
 		send_button = gtk_button_new_with_label(_(" Send "));
-		gtk_signal_connect(GTK_OBJECT(send_button), "clicked",
-				   GTK_SIGNAL_FUNC(send_input), children);
+		g_signal_connect(G_OBJECT(send_button), "clicked",
+				 G_CALLBACK(send_input), children);
 		gtk_box_pack_start(GTK_BOX(input_hbox), send_button, FALSE,
 				   FALSE, 0);
 		gtk_widget_show(send_button);
@@ -1114,12 +1128,13 @@ static void create_io_dialog(Children *children)
 		gtk_widget_grab_focus(entry);
 	}
 
-	gtkut_button_set_create(&hbox, &abort_button, _("Abort"),
-				&close_button, _("Close"), NULL, NULL);
-	gtk_signal_connect(GTK_OBJECT(abort_button), "clicked",
-			GTK_SIGNAL_FUNC(kill_children_cb), children);
-	gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
-			GTK_SIGNAL_FUNC(hide_io_dialog_cb), children);
+	gtkut_stock_button_set_create(&hbox, &abort_button, _("Abort"),
+				      &close_button, GTK_STOCK_CLOSE,
+				      NULL, NULL);
+	g_signal_connect(G_OBJECT(abort_button), "clicked",
+			 G_CALLBACK(kill_children_cb), children);
+	g_signal_connect(G_OBJECT(close_button), "clicked",
+			 G_CALLBACK(hide_io_dialog_cb), children);
 	gtk_widget_show(hbox);
 
 	if (children->nb)
@@ -1149,7 +1164,9 @@ static void catch_status(gpointer data, gint source, GdkInputCondition cond)
 	c = read(source, &buf, 1);
 	debug_print("Child returned %c\n", buf);
 
+#ifdef G_OS_UNIX
 	waitpid(-child_info->pid, NULL, 0);
+#endif
 	childinfo_close_pipes(child_info);
 	child_info->pid = 0;
 
@@ -1196,42 +1213,68 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 static void catch_output(gpointer data, gint source, GdkInputCondition cond)
 {
 	ChildInfo *child_info = (ChildInfo *)data;
-	gint c, i;
+	gint c;
 	gchar buf[BUFFSIZE];
 
 	debug_print("Catching grand child's output.\n");
 	if (child_info->children->action_type &
 	    (ACTION_PIPE_OUT | ACTION_INSERT)
 	    && source == child_info->chld_out) {
-		gboolean is_selection = FALSE;
-		GtkWidget *text = child_info->children->msg_text;
+		GtkTextView *text =
+			GTK_TEXT_VIEW(child_info->children->msg_text);
+		GtkTextBuffer *textbuf = gtk_text_view_get_buffer(text);
+		GtkTextIter iter;
+		GtkTextMark *mark;
+		gint ins_pos;
+
+		mark = gtk_text_buffer_get_insert(textbuf);
+		gtk_text_buffer_get_iter_at_mark(textbuf, &iter, mark);
+		ins_pos = gtk_text_iter_get_offset(&iter);
 
-		if (GTK_EDITABLE(text)->has_selection)
-			is_selection = TRUE;
-		gtk_stext_freeze(GTK_STEXT(text));
 		while (TRUE) {
+			gsize bytes_read = 0, bytes_written = 0;
+			gchar *ret_str;
+
 			c = read(source, buf, sizeof(buf) - 1);
 			if (c == 0)
 				break;
-			gtk_stext_insert(GTK_STEXT(text),
-					 child_info->children->msgfont,
-					 NULL, NULL, buf, c);
+
+			ret_str = g_locale_to_utf8
+				(buf, c, &bytes_read, &bytes_written, NULL);
+			if (ret_str && bytes_written > 0) {
+				gtk_text_buffer_insert
+					(textbuf, &iter, ret_str,
+					 bytes_written);
+				g_free(ret_str);
+			} else
+				gtk_text_buffer_insert(textbuf, &iter, buf, c);
 		}
-		if (is_selection) {
-			/* Using the select_region draws things. Should not.
-			 * so we just change selection position and 
-			 * defere drawing when thawing. Hack?
-			 */
-			GTK_EDITABLE(text)->selection_end_pos =
-					gtk_stext_get_point(GTK_STEXT(text));
+
+		if (child_info->children->is_selection) {
+			GtkTextIter ins;
+
+			gtk_text_buffer_get_iter_at_offset
+				(textbuf, &ins, ins_pos);
+			gtk_text_buffer_select_range(textbuf, &ins, &iter);
 		}
-		gtk_stext_thaw(GTK_STEXT(text));
 	} else {
 		c = read(source, buf, sizeof(buf) - 1);
-		for (i = 0; i < c; i++)
-			g_string_append_c(child_info->output, buf[i]);
-		if (c > 0)
+		if (c > 0) {
+			gsize bytes_read = 0, bytes_written = 0;
+			gchar *ret_str;
+
+			ret_str = g_locale_to_utf8
+				(buf, c, &bytes_read, &bytes_written, NULL);
+			if (ret_str && bytes_written > 0) {
+				g_string_append_len
+					(child_info->output, ret_str,
+					 bytes_written);
+				g_free(ret_str);
+			} else
+				g_string_append_len(child_info->output, buf, c);
+
 			child_info->new_out = TRUE;
+		}
 	}
 	if (c == 0) {
 		if (source == child_info->chld_out) {
