@@ -35,10 +35,6 @@
 
 #include <string.h>
 #include <ctype.h>
-#if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
-#  include <wchar.h>
-#  include <wctype.h>
-#endif
 
 #include "xml.h"
 #include "addr_compl.h"
