@@ -28,15 +28,19 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtkhseparator.h>
+#include <gtk/gtkstock.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "prefs_common.h"
 #include "mgutils.h"
@@ -53,6 +57,7 @@ static struct _AddrBookEdit_Dlg {
 	GtkWidget *window;
 	GtkWidget *name_entry;
 	GtkWidget *file_label;
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *check_btn;
@@ -97,11 +102,12 @@ static gint edit_book_delete_event( GtkWidget *widget, GdkEventAny *event, gbool
 	return TRUE;
 }
 
-static void edit_book_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
+static gboolean edit_book_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void edit_book_file_check( void ) {
@@ -162,18 +168,18 @@ static void addressbook_edit_book_create( gboolean *cancelled ) {
 	GtkWidget *hsbox;
 	gint top;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 450, -1);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 450, -1);
 	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit Addressbook"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_book_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_book_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_book_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_book_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -212,13 +218,14 @@ static void addressbook_edit_book_create( gboolean *cancelled ) {
 
 	/* Status line */
 	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
 	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width( GTK_CONTAINER(hbbox), 0 );
 	gtk_widget_grab_default(ok_btn);
@@ -226,22 +233,23 @@ static void addressbook_edit_book_create( gboolean *cancelled ) {
 	hsep = gtk_hseparator_new();
 	gtk_box_pack_end(GTK_BOX(vbox), hsep, FALSE, FALSE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(name_entry), "focus_in_event",
-			   GTK_SIGNAL_FUNC(edit_book_name_focus), NULL );
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_book_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_book_cancel), cancelled);
-/*	gtk_signal_connect(GTK_OBJECT(file_btn), "clicked", */
-/*			   GTK_SIGNAL_FUNC(edit_book_file_select), NULL); */
-	gtk_signal_connect(GTK_OBJECT(check_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_book_file_check), NULL);
+	g_signal_connect(G_OBJECT(name_entry), "focus_in_event",
+			 G_CALLBACK(edit_book_name_focus), NULL );
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_book_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_book_cancel), cancelled);
+/*	g_signal_connect(G_OBJECT(file_btn), "clicked", */
+/*			 G_CALLBACK(edit_book_file_select), NULL); */
+	g_signal_connect(G_OBJECT(check_btn), "clicked",
+			 G_CALLBACK(edit_book_file_check), NULL);
 
 	gtk_widget_show_all(vbox);
 
 	addrbookedit_dlg.window     = window;
 	addrbookedit_dlg.name_entry = name_entry;
 	addrbookedit_dlg.file_label = file_label;
+	addrbookedit_dlg.hbbox      = hbbox;
 	addrbookedit_dlg.ok_btn     = ok_btn;
 	addrbookedit_dlg.cancel_btn = cancel_btn;
 	addrbookedit_dlg.check_btn  = check_btn;
@@ -261,6 +269,8 @@ AdapterDSource *addressbook_edit_book( AddressIndex *addrIndex, AdapterDSource *
 
 	if (!addrbookedit_dlg.window)
 		addressbook_edit_book_create(&cancelled);
+	gtkut_box_set_reverse_order(GTK_BOX(addrbookedit_dlg.hbbox),
+				    !prefs_common.comply_gnome_hig);
 	gtk_widget_grab_focus(addrbookedit_dlg.ok_btn);
 	gtk_widget_grab_focus(addrbookedit_dlg.name_entry);
 	gtk_widget_show(addrbookedit_dlg.window);
