@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,9 +45,9 @@
 #include "mainwindow.h"
 #include "folderview.h"
 #include "summaryview.h"
-#include "summary_search.h"
+#include "query_search.h"
 #include "inputdialog.h"
-#include "grouplistdialog.h"
+#include "subscribedialog.h"
 #include "foldersel.h"
 #include "manage_window.h"
 #include "alertpanel.h"
@@ -59,11 +59,14 @@
 #include "gtkutils.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
-#include "prefs_filter.h"
 #include "prefs_folder_item.h"
+#include "prefs_search_folder.h"
+#include "filter.h"
 #include "account.h"
+#include "account_dialog.h"
 #include "folder.h"
 #include "inc.h"
+#include "virtual.h"
 
 enum
 {
@@ -104,6 +107,9 @@ static GdkPixbuf *folderopen_pixbuf;
 static GdkPixbuf *foldernoselect_pixbuf;
 static GdkPixbuf *draft_pixbuf;
 static GdkPixbuf *trash_pixbuf;
+static GdkPixbuf *virtual_pixbuf;
+
+static void folderview_set_columns	(FolderView	*folderview);
 
 static void folderview_select_row	(FolderView	*folderview,
 					 GtkTreeIter	*iter);
@@ -116,6 +122,7 @@ static void folderview_append_folder	(FolderView	*folderview,
 
 static void folderview_update_row	(FolderView	*folderview,
 					 GtkTreeIter	*iter);
+static void folderview_update_row_all	(FolderView	*folderview);
 
 static gint folderview_folder_name_compare	(GtkTreeModel	*model,
 						 GtkTreeIter	*a,
@@ -161,6 +168,10 @@ static void folderview_update_tree_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
+static void folderview_update_summary_cb(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void folderview_new_folder_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -238,8 +249,11 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
+	{N_("/Ed_it search condition..."),
+					NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
@@ -257,8 +271,11 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
+	{N_("/Ed_it search condition..."),
+					NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
@@ -267,13 +284,18 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 	{N_("/Su_bscribe to newsgroup..."),
 					NULL, folderview_new_news_group_cb, 0, NULL},
 	{N_("/_Remove newsgroup"),	NULL, folderview_rm_news_group_cb, 0, NULL},
+	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
+	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
+	{N_("/Ed_it search condition..."),
+					NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
@@ -301,8 +323,7 @@ FolderView *folderview_create(void)
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy
 		(GTK_SCROLLED_WINDOW(scrolledwin),
-		 GTK_POLICY_AUTOMATIC,
-		 prefs_common.folderview_vscrollbar_policy);
+		 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
 					    GTK_SHADOW_IN);
 	gtk_widget_set_size_request(scrolledwin,
@@ -312,7 +333,7 @@ FolderView *folderview_create(void)
 	store = gtk_tree_store_new(N_COLS, G_TYPE_STRING, G_TYPE_STRING,
 				   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER,
 				   GDK_TYPE_PIXBUF, GDK_TYPE_PIXBUF,
-				   GDK_TYPE_COLOR, G_TYPE_BOOLEAN);
+				   GDK_TYPE_COLOR, G_TYPE_INT);
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),
 					COL_FOLDER_NAME,
 					folderview_folder_name_compare,
@@ -362,9 +383,8 @@ FolderView *folderview_create(void)
 	gtk_tree_view_column_set_attributes(column, renderer,
 					    "text", COL_FOLDER_NAME,
 					    "foreground-gdk", COL_FOREGROUND,
-					    "weight-set", COL_BOLD,
+					    "weight", COL_BOLD,
 					    NULL);
-	g_object_set(G_OBJECT(renderer), "weight", PANGO_WEIGHT_BOLD, NULL);
 
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(treeview), column);
@@ -486,6 +506,10 @@ FolderView *folderview_create(void)
 	folderview->news_popup   = news_popup;
 	folderview->news_factory = news_factory;
 
+	folderview->display_folder_unread = prefs_common.display_folder_unread;
+
+	folderview_set_columns(folderview);
+
 	gtk_widget_show_all(scrolledwin);
 
 	folderview_list = g_list_append(folderview_list, folderview);
@@ -505,6 +529,18 @@ void folderview_init(FolderView *folderview)
 			 &foldernoselect_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DRAFT, &draft_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_TRASH, &trash_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_GROUP, &virtual_pixbuf);
+}
+
+void folderview_reflect_prefs(FolderView *folderview)
+{
+	folderview_set_columns(folderview);
+	if (folderview->display_folder_unread !=
+	    prefs_common.display_folder_unread) {
+		folderview->display_folder_unread =
+			prefs_common.display_folder_unread;
+		folderview_update_row_all(folderview);
+	}
 }
 
 FolderView *folderview_get(void)
@@ -544,6 +580,22 @@ void folderview_set_all(void)
 		folderview_set((FolderView *)list->data);
 }
 
+static void folderview_set_columns(FolderView *folderview)
+{
+	GtkTreeView *treeview = GTK_TREE_VIEW(folderview->treeview);
+	GtkTreeViewColumn *column;
+
+	column = gtk_tree_view_get_column(treeview, COL_NEW);
+	gtk_tree_view_column_set_visible
+		(column, prefs_common.display_folder_num_columns);
+	column = gtk_tree_view_get_column(treeview, COL_UNREAD);
+	gtk_tree_view_column_set_visible
+		(column, prefs_common.display_folder_num_columns);
+	column = gtk_tree_view_get_column(treeview, COL_TOTAL);
+	gtk_tree_view_column_set_visible
+		(column, prefs_common.display_folder_num_columns);
+}
+
 void folderview_select(FolderView *folderview, FolderItem *item)
 {
 	GtkTreeIter iter;
@@ -876,21 +928,21 @@ static void folderview_rescan_tree(FolderView *folderview, Folder *folder)
 	inc_unlock();
 }
 
-void folderview_check_new(Folder *folder)
+gint folderview_check_new(Folder *folder)
 {
 	FolderItem *item;
 	FolderView *folderview;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gboolean valid;
+	gint prev_new, prev_unread, n_updated = 0;
 
 	folderview = (FolderView *)folderview_list->data;
 	model = GTK_TREE_MODEL(folderview->store);
 
 	if (folder && !FOLDER_IS_LOCAL(folder)) {
-		if (!main_window_toggle_online_if_offline
-			(folderview->mainwin))
-			return;
+		if (!main_window_toggle_online_if_offline(folderview->mainwin))
+			return 0;
 	}
 
 	inc_lock();
@@ -904,16 +956,23 @@ void folderview_check_new(Folder *folder)
 		gtk_tree_model_get(model, &iter,
 				   COL_FOLDER_ITEM, &item, -1);
 		if (!item || !item->path || !item->folder) continue;
+		if (item->stype == F_VIRTUAL) continue;
 		if (item->no_select) continue;
 		if (folder && folder != item->folder) continue;
 		if (!folder && !FOLDER_IS_LOCAL(item->folder)) continue;
 
+		prev_new = item->new;
+		prev_unread = item->unread;
 		folderview_scan_tree_func(item->folder, item, NULL);
 		if (folder_item_scan(item) < 0) {
 			if (folder && !FOLDER_IS_LOCAL(folder))
 				break;
 		}
 		folderview_update_row(folderview, &iter);
+		if (prev_unread < item->unread)
+			n_updated += item->unread - prev_unread;
+		else if (prev_new < item->new)
+			n_updated += item->new - prev_new;
 	}
 
 	gtk_widget_set_sensitive(folderview->treeview, TRUE);
@@ -922,13 +981,69 @@ void folderview_check_new(Folder *folder)
 	statusbar_pop_all();
 
 	folder_write_list();
+
+	return n_updated;
 }
 
-void folderview_check_new_all(void)
+gint folderview_check_new_item(FolderItem *item)
+{
+	Folder *folder;
+	FolderView *folderview;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gint prev_new, prev_unread, n_updated = 0;
+
+	g_return_val_if_fail(item != NULL, 0);
+	g_return_val_if_fail(item->folder != NULL, 0);
+
+	if (!item->path || item->no_select)
+		return 0;
+
+	folderview = (FolderView *)folderview_list->data;
+	model = GTK_TREE_MODEL(folderview->store);
+
+	folder = item->folder;
+
+	if (!FOLDER_IS_LOCAL(folder)) {
+		if (!main_window_toggle_online_if_offline(folderview->mainwin))
+			return 0;
+	}
+
+	if (!gtkut_tree_model_find_by_column_data
+		(model, &iter, NULL, COL_FOLDER_ITEM, item))
+		return 0;
+
+	inc_lock();
+	main_window_lock(folderview->mainwin);
+	gtk_widget_set_sensitive(folderview->treeview, FALSE);
+	GTK_EVENTS_FLUSH();
+
+	prev_new = item->new;
+	prev_unread = item->unread;
+	folderview_scan_tree_func(folder, item, NULL);
+	folder_item_scan(item);
+	folderview_update_row(folderview, &iter);
+	if (prev_unread < item->unread)
+		n_updated = item->unread - prev_unread;
+	else if (prev_new < item->new)
+		n_updated = item->new - prev_new;
+
+	gtk_widget_set_sensitive(folderview->treeview, TRUE);
+	main_window_unlock(folderview->mainwin);
+	inc_unlock();
+	statusbar_pop_all();
+
+	folder_write_list();
+
+	return n_updated;
+}
+
+gint folderview_check_new_all(void)
 {
 	GList *list;
 	GtkWidget *window;
 	FolderView *folderview;
+	gint n_updated = 0;
 
 	folderview = (FolderView *)folderview_list->data;
 
@@ -941,12 +1056,14 @@ void folderview_check_new_all(void)
 	for (; list != NULL; list = list->next) {
 		Folder *folder = list->data;
 
-		folderview_check_new(folder);
+		n_updated += folderview_check_new(folder);
 	}
 
 	gtk_widget_destroy(window);
 	main_window_unlock(folderview->mainwin);
 	inc_unlock();
+
+	return n_updated;
 }
 
 static gboolean folderview_search_new_recursive(GtkTreeModel *model,
@@ -1007,6 +1124,8 @@ static gboolean folderview_search_unread_recursive(GtkTreeModel *model,
 	if (iter) {
 		gtk_tree_model_get(model, iter, COL_FOLDER_ITEM, &item, -1);
 		if (item) {
+			if (item->stype == F_TRASH)
+				return FALSE;
 			if (item->unread > 0 ||
 			    (item->stype == F_QUEUE && item->total > 0))
 				return TRUE;
@@ -1056,7 +1175,8 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 	gchar *name, *str;
 	gchar new_s[11], unread_s[11], total_s[11];
 	gboolean add_unread_mark;
-	gboolean use_bold, use_color;
+	gboolean use_color;
+	PangoWeight weight = PANGO_WEIGHT_NORMAL;
 	GdkColor *foreground = NULL;
 
 	gtk_tree_model_get(model, iter, COL_FOLDER_ITEM, &item, -1);
@@ -1093,6 +1213,19 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 				!strcmp2(item->name, DRAFT_DIR) ? _("Drafts") :
 				item->name);
 		break;
+#if 0
+	case F_JUNK:
+		pixbuf = folder_pixbuf;
+		open_pixbuf = folderopen_pixbuf;
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, JUNK_DIR) ? _("Junk") :
+				item->name);
+		break;
+#endif
+	case F_VIRTUAL:
+		pixbuf = open_pixbuf = virtual_pixbuf;
+		name = g_strdup(item->name);
+		break;
 	default:
 		if (item->no_select) {
 			pixbuf = open_pixbuf = foldernoselect_pixbuf;
@@ -1135,13 +1268,13 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 	gtk_tree_path_free(path);
 
 	if (item->stype == F_QUEUE && item->total > 0 &&
-	    prefs_common.display_folder_unread) {
+	    folderview->display_folder_unread) {
 		str = g_strdup_printf("%s (%d%s)", name, item->total,
 				      add_unread_mark ? "+" : "");
 		g_free(name);
 		name = str;
 	} else if ((item->unread > 0 || add_unread_mark) &&
-		   prefs_common.display_folder_unread) {
+		   folderview->display_folder_unread) {
 		if (item->unread > 0)
 			str = g_strdup_printf("%s (%d%s)", name, item->unread,
 					      add_unread_mark ? "+" : "");
@@ -1163,13 +1296,16 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 
 	if (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
 	    item->stype == F_TRASH) {
-		use_bold = use_color = FALSE;
+		use_color = FALSE;
 	} else if (item->stype == F_QUEUE) {
 		/* highlight queue folder if there are any messages */
-		use_bold = use_color = (item->total > 0);
+		use_color = (item->total > 0);
+		if (item->total > 0)
+			weight = PANGO_WEIGHT_BOLD;
 	} else {
 		/* if unread messages exist, print with bold font */
-		use_bold = (item->unread > 0) || add_unread_mark;
+		if ((item->unread > 0) || add_unread_mark)
+			weight = PANGO_WEIGHT_BOLD;
 		/* if new messages exist, print with colored letter */
 		use_color =
 			(item->new > 0) ||
@@ -1191,7 +1327,7 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 			   COL_PIXBUF, pixbuf,
 			   COL_PIXBUF_OPEN, open_pixbuf,
 			   COL_FOREGROUND, foreground,
-			   COL_BOLD, use_bold,
+			   COL_BOLD, weight,
 			   -1);
 	g_free(name);
 
@@ -1202,6 +1338,20 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 		folderview_update_row(folderview, &parent);
 }
 
+static void folderview_update_row_all(FolderView *folderview)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
+	GtkTreeIter iter;
+	gboolean valid;
+
+	valid = gtk_tree_model_get_iter_first(model, &iter);
+
+	while (valid) {
+		folderview_update_row(folderview, &iter);
+		valid = gtkut_tree_model_next(model, &iter);
+	}
+}
+
 void folderview_update_item(FolderItem *item, gboolean update_summary)
 {
 	FolderView *folderview;
@@ -1431,6 +1581,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	FolderItem *item = NULL;
 	Folder *folder;
 	GtkWidget *popup;
+	GtkItemFactory *ifactory;
 	GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
 	GtkTreeIter iter;
 	gboolean new_folder      = FALSE;
@@ -1440,6 +1591,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	gboolean empty_trash     = FALSE;
 	gboolean download_msg    = FALSE;
 	gboolean update_tree     = FALSE;
+	gboolean update_summary  = FALSE;
 	gboolean rescan_tree     = FALSE;
 	gboolean remove_tree     = FALSE;
 	gboolean search_folder   = FALSE;
@@ -1461,15 +1613,14 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 
 	if (folderview->mainwin->lock_count == 0) {
 		new_folder = TRUE;
+		folder_property = TRUE;
+		search_folder = TRUE;
 		if (item->parent == NULL) {
 			update_tree = remove_tree = TRUE;
-			if (folder->account)
-				folder_property = TRUE;
 		} else {
-			folder_property = TRUE;
-			if (gtkut_tree_row_reference_equal(folderview->selected,
-							   folderview->opened))
-				search_folder = TRUE;
+			if (gtkut_tree_row_reference_equal
+				(folderview->selected, folderview->opened))
+				update_summary = TRUE;
 		}
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
@@ -1478,60 +1629,92 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 				rename_folder = delete_folder = TRUE;
 				if (folder->klass->move_folder)
 					move_folder = TRUE;
-			} else if (item->stype == F_TRASH)
-				empty_trash = TRUE;
+			} else if (item->stype == F_TRASH) {
+				if (item->total > 0)
+					empty_trash = TRUE;
+			}
 		} else if (FOLDER_TYPE(folder) == F_NEWS) {
 			if (item->parent != NULL)
 				delete_folder = TRUE;
 		}
+		if (item->stype == F_VIRTUAL) {
+			new_folder = FALSE;
+			rename_folder = delete_folder = TRUE;
+		}
 		if (FOLDER_TYPE(folder) == F_IMAP ||
 		    FOLDER_TYPE(folder) == F_NEWS) {
-			if (item->parent != NULL && item->no_select == FALSE)
+			if (item->parent != NULL && item->no_select == FALSE &&
+			    item->stype != F_VIRTUAL)
 				download_msg = TRUE;
 		}
 	}
 
-#define SET_SENS(factory, name, sens) \
-	menu_set_sensitive(folderview->factory, name, sens)
+#define SET_SENS(factory, name, sens)				\
+{								\
+	GtkWidget *widget;					\
+	widget = gtk_item_factory_get_item(factory, name);	\
+	if (widget)						\
+		gtk_widget_set_sensitive(widget, sens);		\
+}
+
+#define SET_VISIBILITY(factory, name, visible)			\
+{								\
+	GtkWidget *widget;					\
+	widget = gtk_item_factory_get_item(factory, name);	\
+	if (widget) {						\
+		if (visible)					\
+			gtk_widget_show(widget);		\
+		else						\
+			gtk_widget_hide(widget);		\
+	}							\
+}
 
 	if (FOLDER_IS_LOCAL(folder)) {
 		popup = folderview->mail_popup;
-		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
-		SET_SENS(mail_factory, "/Create new folder...", new_folder);
-		SET_SENS(mail_factory, "/Rename folder...", rename_folder);
-		SET_SENS(mail_factory, "/Move folder...", move_folder);
-		SET_SENS(mail_factory, "/Delete folder", delete_folder);
-		SET_SENS(mail_factory, "/Empty trash", empty_trash);
-		SET_SENS(mail_factory, "/Check for new messages", update_tree);
-		SET_SENS(mail_factory, "/Rebuild folder tree", rescan_tree);
-		SET_SENS(mail_factory, "/Search messages...", search_folder);
-		SET_SENS(mail_factory, "/Properties...", folder_property);
+		ifactory = folderview->mail_factory;
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
 		popup = folderview->imap_popup;
-		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
-		SET_SENS(imap_factory, "/Create new folder...", new_folder);
-		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
-		SET_SENS(imap_factory, "/Move folder...", move_folder);
-		SET_SENS(imap_factory, "/Delete folder", delete_folder);
-		SET_SENS(imap_factory, "/Empty trash", empty_trash);
-		SET_SENS(imap_factory, "/Download", download_msg);
-		SET_SENS(imap_factory, "/Check for new messages", update_tree);
-		SET_SENS(imap_factory, "/Rebuild folder tree", rescan_tree);
-		SET_SENS(imap_factory, "/Search messages...", search_folder);
-		SET_SENS(imap_factory, "/Properties...", folder_property);
+		ifactory = folderview->imap_factory;
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
 		popup = folderview->news_popup;
-		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
-		SET_SENS(news_factory, "/Subscribe to newsgroup...", new_folder);
-		SET_SENS(news_factory, "/Remove newsgroup", delete_folder);
-		SET_SENS(news_factory, "/Download", download_msg);
-		SET_SENS(news_factory, "/Check for new messages", update_tree);
-		SET_SENS(news_factory, "/Search messages...", search_folder);
-		SET_SENS(news_factory, "/Properties...", folder_property);
+		ifactory = folderview->news_factory;
 	} else
 		return FALSE;
 
+	menu_set_insensitive_all(GTK_MENU_SHELL(popup));
+
+	SET_SENS(ifactory, "/Create new folder...", new_folder);
+	SET_SENS(ifactory, "/Rename folder...", rename_folder);
+	SET_SENS(ifactory, "/Move folder...", move_folder);
+	SET_SENS(ifactory, "/Delete folder", delete_folder);
+	SET_SENS(ifactory, "/Empty trash", empty_trash);
+	SET_SENS(ifactory, "/Download", download_msg);
+	SET_SENS(ifactory, "/Check for new messages", update_tree);
+	SET_SENS(ifactory, "/Rebuild folder tree", rescan_tree);
+	SET_SENS(ifactory, "/Update summary", update_summary);
+	SET_SENS(ifactory, "/Search messages...", search_folder);
+	SET_SENS(ifactory, "/Edit search condition...", search_folder);
+	SET_SENS(ifactory, "/Properties...", folder_property);
+
+	if (FOLDER_TYPE(folder) == F_NEWS) {
+		SET_SENS(ifactory, "/Subscribe to newsgroup...", new_folder);
+		SET_SENS(ifactory, "/Remove newsgroup", delete_folder);
+		SET_VISIBILITY(ifactory, "/Remove newsgroup",
+			       item->stype != F_VIRTUAL);
+		SET_VISIBILITY(ifactory, "/Rename folder...",
+			       item->stype == F_VIRTUAL);
+		SET_VISIBILITY(ifactory, "/Delete folder",
+			       item->stype == F_VIRTUAL);
+	}
+
+	SET_VISIBILITY(ifactory, "/Search messages...",
+		       item->stype != F_VIRTUAL);
+	SET_VISIBILITY(ifactory, "/Edit search condition...",
+		       item->stype == F_VIRTUAL);
+
+
 #undef SET_SENS
+#undef SET_VISIBILITY
 
 	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
 		       event->button, event->time);
@@ -1605,6 +1788,7 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 	switch (event->keyval) {
 	case GDK_Return:
+	case GDK_KP_Enter:
 		if (folderview->selected) {
 			folderview_select_row_ref(folderview,
 						  folderview->selected);
@@ -1612,6 +1796,7 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		return TRUE;
 		break;
 	case GDK_space:
+	case GDK_KP_Space:
 		if (folderview->selected) {
 			if (folderview->opened)
 				opened = gtk_tree_row_reference_get_path
@@ -1857,6 +2042,8 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 	item = folderview_get_selected_item(folderview);
 	if (!item)
 		return;
+	if (item->stype == F_VIRTUAL)
+		return;
 
 	g_return_if_fail(item->folder != NULL);
 
@@ -1903,6 +2090,17 @@ static void folderview_update_tree_cb(FolderView *folderview, guint action,
 		folderview_rescan_tree(folderview, item->folder);
 }
 
+static void folderview_update_summary_cb(FolderView *folderview, guint action,
+					 GtkWidget *widget)
+{
+	if (!folderview->summaryview->folder_item)
+		return;
+
+	GTK_EVENTS_FLUSH();
+	summary_show(folderview->summaryview,
+		     folderview->summaryview->folder_item, TRUE);
+}
+
 static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget)
 {
@@ -1985,18 +2183,24 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+
 	name = trim_string(item->name, 32);
 	message = g_strdup_printf(_("Input new name for `%s':"), name);
 	new_folder = input_dialog(_("Rename folder"), message,
 				  g_basename(item->path));
 	g_free(message);
 	g_free(name);
-	if (!new_folder) return;
-	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
+	if (!new_folder) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
+	AUTORELEASE_STR(new_folder, {g_free(new_folder); gtk_tree_path_free(sel_path); return;});
 
 	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
+		gtk_tree_path_free(sel_path);
 		return;
 	}
 
@@ -2004,26 +2208,38 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 		name = trim_string(new_folder, 32);
 		alertpanel_error(_("The folder `%s' already exists."), name);
 		g_free(name);
+		gtk_tree_path_free(sel_path);
 		return;
 	}
 
 	Xstrdup_a(old_path, item->path, {g_free(new_folder); return;});
 	old_id = folder_item_get_identifier(item);
 
-	if (item->folder->klass->rename_folder(item->folder, item,
-					       new_folder) < 0) {
+	if (item->stype == F_VIRTUAL) {
+		if (virtual_get_class()->rename_folder(item->folder, item,
+						       new_folder) < 0) {
+			alertpanel_error(_("Can't rename the folder '%s'."),
+					 item->name);
+			g_free(old_id);
+			gtk_tree_path_free(sel_path);
+			return;
+		}
+	} else if (item->folder->klass->rename_folder(item->folder, item,
+						      new_folder) < 0) {
+		alertpanel_error(_("Can't rename the folder '%s'."),
+				 item->name);
 		g_free(old_id);
+		gtk_tree_path_free(sel_path);
 		return;
 	}
 
 	if (folder_get_default_folder() == item->folder)
-		prefs_filter_rename_path(old_path, item->path);
+		filter_list_rename_path(old_path, item->path);
 	new_id = folder_item_get_identifier(item);
-	prefs_filter_rename_path(old_id, new_id);
+	filter_list_rename_path(old_id, new_id);
 	g_free(old_id);
 	g_free(new_id);
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	if (folderview->opened)
 		open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	if (sel_path) {
@@ -2064,14 +2280,16 @@ static void folderview_move_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+	g_return_if_fail(sel_path != NULL);
+
 	new_parent = foldersel_folder_sel(item->folder, FOLDER_SEL_MOVE_FOLDER,
 					  NULL);
-	if (!new_parent)
-		return;
-	if (new_parent->folder != item->folder)
-		return;
-	if (new_parent == item->parent)
+	if (!new_parent || new_parent->folder != item->folder ||
+	    new_parent == item->parent || new_parent->stype == F_VIRTUAL) {
+		gtk_tree_path_free(sel_path);
 		return;
+	}
 
 	old_path = g_strdup(item->path);
 	old_id = folder_item_get_identifier(item);
@@ -2081,21 +2299,20 @@ static void folderview_move_folder_cb(FolderView *folderview, guint action,
 		alertpanel_error(_("Can't move the folder `%s'."), item->name);
 		g_free(old_id);
 		g_free(old_path);
+		gtk_tree_path_free(sel_path);
 		return;
 	}
 
 	if (folder_get_default_folder() == item->folder)
-		prefs_filter_rename_path(old_path, item->path);
+		filter_list_rename_path(old_path, item->path);
 	new_id = folder_item_get_identifier(item);
-	prefs_filter_rename_path(old_id, new_id);
+	filter_list_rename_path(old_id, new_id);
 	g_free(new_id);
 	g_free(old_id);
 	g_free(old_path);
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	if (folderview->opened)
 		open_path = gtk_tree_row_reference_get_path(folderview->opened);
-	g_return_if_fail(sel_path != NULL);
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store), &iter,
 				sel_path);
 	if (sel_path && open_path &&
@@ -2136,25 +2353,38 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+	g_return_if_fail(sel_path != NULL);
+
 	folder = item->folder;
 
 	name = trim_string(item->name, 32);
-	AUTORELEASE_STR(name, {g_free(name); return;});
-	message = g_strdup_printf
-		(_("All folders and messages under `%s' will be permanently deleted.\n"
-		   "Recovery will not be possible.\n\n"
-		   "Do you really want to delete?"), name);
-	avalue = alertpanel_full(_("Delete folder"), message,
-				 ALERT_WARNING, G_ALERTALTERNATE, FALSE,
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+	AUTORELEASE_STR(name, {g_free(name); gtk_tree_path_free(sel_path); return;});
+	if (item->stype == F_VIRTUAL) {
+		message = g_strdup_printf
+			(_("Delete the search folder '%s' ?\n"
+			   "The real messages are not deleted."), name);
+		avalue = alertpanel_full(_("Delete search folder"), message,
+					 ALERT_WARNING, G_ALERTALTERNATE, FALSE,
+					 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+	} else {
+		message = g_strdup_printf
+			(_("All folders and messages under '%s' will be permanently deleted.\n"
+			   "Recovery will not be possible.\n\n"
+			   "Do you really want to delete?"), name);
+		avalue = alertpanel_full(_("Delete folder"), message,
+					 ALERT_WARNING, G_ALERTALTERNATE, FALSE,
+					 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+	}
 	g_free(message);
-	if (avalue != G_ALERTDEFAULT) return;
+	if (avalue != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
 
 	Xstrdup_a(old_path, item->path, return);
 	old_id = folder_item_get_identifier(item);
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
-	g_return_if_fail(sel_path != NULL);
 	if (folderview->opened)
 		open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store), &iter,
@@ -2169,15 +2399,22 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	gtk_tree_path_free(open_path);
 	gtk_tree_path_free(sel_path);
 
-	if (folder->klass->remove_folder(folder, item) < 0) {
-		alertpanel_error(_("Can't remove the folder `%s'."), name);
+	if (item->stype == F_VIRTUAL) {
+		if (virtual_get_class()->remove_folder(folder, item) < 0) {
+			alertpanel_error(_("Can't remove the folder '%s'."),
+					 name);
+			g_free(old_id);
+			return;
+		}
+	} else if (folder->klass->remove_folder(folder, item) < 0) {
+		alertpanel_error(_("Can't remove the folder '%s'."), name);
 		g_free(old_id);
 		return;
 	}
 
 	if (folder_get_default_folder() == folder)
-		prefs_filter_delete_path(old_path);
-	prefs_filter_delete_path(old_id);
+		filter_list_delete_path(old_path);
+	filter_list_delete_path(old_id);
 	g_free(old_id);
 
 	gtk_tree_store_remove(folderview->store, &iter);
@@ -2204,15 +2441,19 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	if (folder->trash != item) return;
 	if (item->stype != F_TRASH) return;
 
-	if (alertpanel(_("Empty trash"), _("Empty all messages in trash?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+
+	if (alertpanel(_("Empty trash"),
+		       _("Delete all messages in the trash folder?"),
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
 		return;
+	}
 
 	procmsg_empty_trash(folder->trash);
 	statusbar_pop_all();
 	folderview_update_item(folder->trash, TRUE);
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	if (open_path && sel_path &&
 	    gtk_tree_path_compare(open_path, sel_path) == 0)
@@ -2238,6 +2479,8 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder != NULL);
 	if (item->parent) return;
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+
 	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf
 		(_("Really remove the mailbox `%s' ?\n"
@@ -2247,7 +2490,10 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 				 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
 	g_free(message);
 	g_free(name);
-	if (avalue != G_ALERTDEFAULT) return;
+	if (avalue != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
 
 	if (folderview->summaryview->folder_item &&
 	    folderview->summaryview->folder_item->folder == item->folder) {
@@ -2257,7 +2503,6 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	}
 	folder_destroy(item->folder);
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	if (sel_path) {
 		gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store),
 					&iter, sel_path);
@@ -2287,6 +2532,8 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+
 	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete IMAP4 account `%s'?"), name);
 	avalue = alertpanel_full(_("Delete IMAP4 account"), message,
@@ -2295,7 +2542,10 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	g_free(message);
 	g_free(name);
 
-	if (avalue != G_ALERTDEFAULT) return;
+	if (avalue != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
 
 	if (folderview->summaryview->folder_item &&
 	    folderview->summaryview->folder_item->folder == item->folder) {
@@ -2307,8 +2557,8 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	account = item->folder->account;
 	folder_destroy(item->folder);
 	account_destroy(account);
+	account_write_config_all();
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	if (sel_path) {
 		gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store),
 					&iter, sel_path);
@@ -2354,7 +2604,7 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 
 	gtk_tree_model_get(model, &root, COL_FOLDER_ITEM, &rootitem, -1);
 
-	new_subscr = grouplist_dialog(folder);
+	new_subscr = subscribe_dialog(folder);
 
 	/* remove unsubscribed newsgroups */
 	for (gnode = folder->node->children; gnode != NULL; ) {
@@ -2426,10 +2676,18 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 	if (!item)
 		return;
 
+	if (item->stype == F_VIRTUAL) {
+		folderview_delete_folder_cb(folderview, 0, widget);
+		return;
+	}
+
 	g_return_if_fail(item->folder != NULL);
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+	g_return_if_fail(sel_path != NULL);
+
 	name = trim_string_before(item->path, 32);
 	message = g_strdup_printf(_("Really delete newsgroup `%s'?"), name);
 	avalue = alertpanel_full(_("Delete newsgroup"), message,
@@ -2437,10 +2695,11 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 				 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
 	g_free(message);
 	g_free(name);
-	if (avalue != G_ALERTDEFAULT) return;
+	if (avalue != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
-	g_return_if_fail(sel_path != NULL);
 	if (folderview->opened)
 		open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store), &iter,
@@ -2478,6 +2737,8 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
+	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
+
 	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete news account `%s'?"), name);
 	avalue = alertpanel_full(_("Delete news account"), message,
@@ -2486,7 +2747,10 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	g_free(message);
 	g_free(name);
 
-	if (avalue != G_ALERTDEFAULT) return;
+	if (avalue != G_ALERTDEFAULT) {
+		gtk_tree_path_free(sel_path);
+		return;
+	}
 
 	if (folderview->summaryview->folder_item &&
 	    folderview->summaryview->folder_item->folder == item->folder) {
@@ -2498,8 +2762,8 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	account = item->folder->account;
 	folder_destroy(item->folder);
 	account_destroy(account);
+	account_write_config_all();
 
-	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 	if (sel_path) {
 		gtk_tree_model_get_iter(GTK_TREE_MODEL(folderview->store),
 					&iter, sel_path);
@@ -2515,7 +2779,35 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 static void folderview_search_cb(FolderView *folderview, guint action,
 				 GtkWidget *widget)
 {
-	summary_search(folderview->summaryview);
+	FolderItem *item;
+
+	item = folderview_get_selected_item(folderview);
+	if (!item)
+		return;
+
+	if (item->stype == F_VIRTUAL) {
+		GtkTreePath *sel_path, *open_path;
+
+		sel_path = gtk_tree_row_reference_get_path
+			(folderview->selected);
+		open_path = gtk_tree_row_reference_get_path(folderview->opened);
+
+		if (prefs_search_folder_open(item)) {
+			if (sel_path && open_path &&
+			    gtk_tree_path_compare(open_path, sel_path) == 0) {
+				GtkTreeRowReference *row;
+				row = gtk_tree_row_reference_copy(folderview->opened);
+				folderview_unselect(folderview);
+				summary_clear_all(folderview->summaryview);
+				folderview_select_row_ref(folderview, row);
+				gtk_tree_row_reference_free(row);
+			}
+		}
+
+		gtk_tree_path_free(open_path);
+		gtk_tree_path_free(sel_path);
+	} else
+		query_search(item);
 }
 
 static void folderview_property_cb(FolderView *folderview, guint action,
@@ -2597,7 +2889,9 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		gtk_tree_model_get_iter(model, &iter, path);
 		gtk_tree_model_get(model, &iter, COL_FOLDER_ITEM, &item, -1);
 		src_item = folderview->summaryview->folder_item;
-		if (src_item && src_item != item)
+		if (src_item && src_item != item &&
+		    src_item->stype != F_QUEUE && item->stype != F_QUEUE &&
+		    item->stype != F_VIRTUAL)
 			acceptable = FOLDER_ITEM_CAN_ADD(item);
 	} else
 		remove_auto_expand_timeout(folderview);
@@ -2624,6 +2918,19 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		}
 	}
 
+#ifdef G_OS_WIN32
+	/* Win32 hack: somehow context->actions is not properly set on Win32 */
+	{
+		GdkWindow *rootwin;
+		GdkModifierType state;
+
+		rootwin = gtk_widget_get_root_window(widget);
+		gdk_window_get_pointer(rootwin, NULL, NULL, &state);
+		if ((state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) == 0)
+			context->actions = GDK_ACTION_MOVE | GDK_ACTION_COPY;
+	}
+#endif
+
 	if (acceptable) {
 		if ((context->actions & GDK_ACTION_MOVE) != 0 &&
 		    FOLDER_ITEM_CAN_ADD(src_item))
@@ -2680,7 +2987,9 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	gtk_tree_model_get(model, &iter, COL_FOLDER_ITEM, &item, -1);
 	src_item = folderview->summaryview->folder_item;
 
-	if (FOLDER_ITEM_CAN_ADD(item) && src_item && src_item != item) {
+	if (FOLDER_ITEM_CAN_ADD(item) && src_item && src_item != item &&
+	    src_item->stype != F_QUEUE && item->stype != F_QUEUE &&
+	    item->stype != F_VIRTUAL) {
 		if ((context->actions & GDK_ACTION_MOVE) != 0 &&
 		    FOLDER_ITEM_CAN_ADD(src_item)) {
 			summary_move_selected_to(folderview->summaryview, item);
