@@ -1,6 +1,6 @@
 /* GtkSHRuler
  *
- *  Copyright (C) 2000 Alfons Hoogervorst
+ *  Copyright (C) 2000-2005 Alfons Hoogervorst & The Sylpheed Claws Team
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@
  * last parameter has the distance between two ticks (which is
  * the width of the fixed font character!
  * 
- * -- Alfons (alfons@proteus.demon.nl)
+ * -- Alfons
  */
 
 #include <math.h>
@@ -43,33 +43,33 @@
 
 static void gtk_shruler_class_init   	(GtkSHRulerClass *klass);
 static void gtk_shruler_init         	(GtkSHRuler      *hruler);
-#if 0
-static gint gtk_shruler_motion_notify	(GtkWidget       *widget,
-					 GdkEventMotion  *event);
-#endif
 static void gtk_shruler_draw_ticks 	(GtkRuler        *ruler);
 #if 0
 static void gtk_shruler_draw_pos      	(GtkRuler        *ruler);
 #endif
 
-guint
+GType
 gtk_shruler_get_type(void)
 {
-	static guint shruler_type = 0;
+	static GType shruler_type = 0;
 
   	if ( !shruler_type ) {
-   		static const GtkTypeInfo shruler_info = {
-			"GtkSHRuler",
-			sizeof (GtkSHRuler),
+   		static const GTypeInfo shruler_info = {
 			sizeof (GtkSHRulerClass),
-			(GtkClassInitFunc) gtk_shruler_class_init,
-			(GtkObjectInitFunc) gtk_shruler_init,
-			/* reserved_1 */ NULL,
-        	/* reserved_2 */ NULL,
-        	(GtkClassInitFunc) NULL,
-     	 };
+
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+
+			(GClassInitFunc) gtk_shruler_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,	/* class_data */
+
+			sizeof (GtkSHRuler),
+			0,	/* n_preallocs */
+			(GInstanceInitFunc) gtk_shruler_init,
+		};
 		/* inherit from GtkHRuler */
-      	shruler_type = gtk_type_unique( gtk_hruler_get_type (), &shruler_info );
+		shruler_type = g_type_register_static (GTK_TYPE_HRULER, "GtkSHRuler", &shruler_info, (GTypeFlags)0);
 	}
 	return shruler_type;
 }
@@ -103,33 +103,22 @@ gtk_shruler_init (GtkSHRuler * shruler)
 	GtkWidget * widget;
 	
 	widget = GTK_WIDGET (shruler);
-	widget->requisition.width = widget->style->klass->xthickness * 2 + 1;
-	widget->requisition.height = widget->style->klass->ythickness * 2 + RULER_HEIGHT;
+	widget->requisition.width = widget->style->xthickness * 2 + 1;
+	widget->requisition.height = widget->style->ythickness * 2 + RULER_HEIGHT;
 }
 
 
 GtkWidget*
 gtk_shruler_new(void)
 {
-	return GTK_WIDGET( gtk_type_new( gtk_shruler_get_type() ) );
+	return GTK_WIDGET( g_object_new( gtk_shruler_get_type(), NULL ) );
 }
 
-#if 0
-static gint
-gtk_shruler_motion_notify(GtkWidget      *widget,
-			  GdkEventMotion *event)
-{
-	/* I could have perhaps set this to NULL */
-	return FALSE;
-}
-#endif
-
 static void
 gtk_shruler_draw_ticks(GtkRuler *ruler)
 {
 	GtkWidget *widget;
 	GdkGC *gc, *bg_gc;
-	GdkFont *font;
 	gint i;
 	gint width, height;
 	gint xthickness;
@@ -146,10 +135,9 @@ gtk_shruler_draw_ticks(GtkRuler *ruler)
 	
 	gc = widget->style->fg_gc[GTK_STATE_NORMAL];
 	bg_gc = widget->style->bg_gc[GTK_STATE_NORMAL];
-	font = widget->style->font;
 
-	xthickness = widget->style->klass->xthickness;
-	ythickness = widget->style->klass->ythickness;
+	xthickness = widget->style->xthickness;
+	ythickness = widget->style->ythickness;
 
 	width = widget->allocation.width;
 	height = widget->allocation.height - ythickness * 2;
@@ -168,26 +156,34 @@ gtk_shruler_draw_ticks(GtkRuler *ruler)
 		       height + ythickness);
 #endif
 
-	/* assume ruler->max_size has the char width */    
-	/* i is increment of char_width,  pos is label number */
+	/* assume ruler->max_size has the char width */
+	/* i is increment of char_width,  pos is label number
+	 * y position is based on height of widget itself */
 	for ( i = 0, pos = 0; i < widget->allocation.width - xthickness; i += ruler->max_size, pos++ ) {	
-		int length = ythickness / 2;
+		gint length = height / 8;
 	
-		if ( pos % 10 == 0 ) length = ( 4 * ythickness );
-		else if (pos % 5 == 0 ) length = ( 2 * ythickness );
+		if ( pos % 10 == 0 ) length = ( 2 * height / 3 );
+		else if ( pos % 5 == 0 ) length = ( height / 3 );
 		
 		gdk_draw_line(ruler->backing_store, gc,
-			      i, height + ythickness, 
+			      i, height + ythickness,
 			      i, height - length);			
 		
 		if ( pos % 10 == 0 ) {
-			char buf[8];
+			gchar buf[8];
+			PangoLayout *layout;
+
 			/* draw label */
-			sprintf(buf, "%d", (int) pos);
-			gdk_draw_string(ruler->backing_store, font, gc,
-					i + 2, ythickness + font->ascent - 1,
-					buf);
-	    }
+			g_snprintf(buf, sizeof buf, "%d", pos);
+
+			layout = gtk_widget_create_pango_layout
+				(GTK_WIDGET(ruler), buf);
+
+			gdk_draw_layout(ruler->backing_store, gc, i + 2,
+					0, layout);
+
+			g_object_unref(layout);
+		}
 	}
 }
 
