@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,18 +31,18 @@
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkctree.h>
+#include <gtk/gtktreemodel.h>
+#include <gtk/gtktreeview.h>
 #include <gtk/gtkcombo.h>
-#include <gtk/gtktext.h>
+#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtktextview.h>
 #include <gtk/gtkitemfactory.h>
+#include <gtk/gtkstock.h>
+
 #include <stdlib.h>
-#if HAVE_WCHAR_H
-#  include <wchar.h>
-#endif
 
 typedef struct _ComboButton	ComboButton;
 
-#include "gtkstext.h"
-
 struct _ComboButton
 {
 	GtkWidget *arrow;
@@ -79,18 +79,20 @@ struct _ComboButton
 	(GTK_CTREE_ROW(node)->row.data)
 
 #define GTKUT_CTREE_REFRESH(clist) \
-	GTK_CLIST_CLASS(GTK_OBJECT(clist)->klass)->refresh(clist)
+	GTK_CLIST_GET_CLASS(clist)->refresh(clist)
 
-gint gtkut_get_font_width		(GdkFont	*font);
-gint gtkut_get_font_height		(GdkFont	*font);
-
-GdkFont *gtkut_font_load		(const gchar	*fontset_name);
-GdkFont *gtkut_font_load_from_fontset	(const gchar	*fontset_name);
+gboolean gtkut_get_font_size		(GtkWidget	*widget,
+					 gint		*width,
+					 gint		*height);
+PangoFontDescription *gtkut_get_default_font_desc
+					(void);
+void gtkut_widget_set_small_font_size	(GtkWidget	*widget);
 
 void gtkut_convert_int_to_gdk_color	(gint		 rgbvalue,
 					 GdkColor	*color);
 
-void gtkut_button_set_create		(GtkWidget	**bbox,
+void gtkut_stock_button_set_set_reverse	(gboolean	 reverse);
+void gtkut_stock_button_set_create	(GtkWidget	**bbox,
 					 GtkWidget	**button1,
 					 const gchar	 *label1,
 					 GtkWidget	**button2,
@@ -98,34 +100,69 @@ void gtkut_button_set_create		(GtkWidget	**bbox,
 					 GtkWidget	**button3,
 					 const gchar	 *label3);
 
+void gtkut_box_set_reverse_order	(GtkBox		*box,
+					 gboolean	 reverse);
+
 ComboButton *gtkut_combo_button_create	(GtkWidget		*button,
 					 GtkItemFactoryEntry	*entries,
 					 gint			 n_entries,
 					 const gchar		*path,
 					 gpointer		 data);
 
-void gtkut_ctree_node_move_if_on_the_edge
-					(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
+/* CTree functions */
+
 gint gtkut_ctree_get_nth_from_node	(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
-GtkCTreeNode *gtkut_ctree_node_next	(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
-GtkCTreeNode *gtkut_ctree_node_prev	(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
-gboolean gtkut_ctree_node_is_selected	(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
-GtkCTreeNode *gtkut_ctree_find_collapsed_parent
-					(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
-void gtkut_ctree_expand_parent_all	(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
 void gtkut_ctree_set_focus_row		(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
-
 void gtkut_clist_set_focus_row		(GtkCList	*clist,
 					 gint		 row);
 
+/* TreeView functions */
+
+gboolean gtkut_tree_model_next		(GtkTreeModel	*model,
+					 GtkTreeIter	*iter);
+gboolean gtkut_tree_model_prev		(GtkTreeModel	*model,
+					 GtkTreeIter	*iter);
+
+gboolean gtkut_tree_model_get_iter_last	(GtkTreeModel	*model,
+					 GtkTreeIter	*iter);
+
+gboolean gtkut_tree_model_find_by_column_data
+					(GtkTreeModel	*model,
+					 GtkTreeIter	*iter,
+					 GtkTreeIter	*start,
+					 gint		 col,
+					 gpointer	 data);
+
+void gtkut_tree_model_foreach		(GtkTreeModel		*model,
+					 GtkTreeIter		*start,
+					 GtkTreeModelForeachFunc func,
+					 gpointer		 user_data);
+
+gboolean gtkut_tree_row_reference_get_iter
+					(GtkTreeModel		*model,
+					 GtkTreeRowReference	*ref,
+					 GtkTreeIter		*iter);
+gboolean gtkut_tree_row_reference_equal	(GtkTreeRowReference	*ref1,
+					 GtkTreeRowReference	*ref2);
+
+void gtkut_tree_sortable_unset_sort_column_id
+					(GtkTreeSortable	*sortable);
+
+gboolean gtkut_tree_view_find_collapsed_parent
+					(GtkTreeView	*treeview,
+					 GtkTreeIter	*parent,
+					 GtkTreeIter	*iter);
+void gtkut_tree_view_expand_parent_all	(GtkTreeView	*treeview,
+					 GtkTreeIter	*iter);
+
+void gtkut_tree_view_vertical_autoscroll(GtkTreeView	*treeview);
+
+void gtkut_tree_view_scroll_to_cell	(GtkTreeView	*treeview,
+					 GtkTreePath	*path,
+					 gboolean	 align_center);
+
 void gtkut_combo_set_items		(GtkCombo	*combo,
 					 const gchar	*str1, ...);
 
@@ -135,36 +172,45 @@ void gtkut_editable_disable_im		(GtkEditable	*editable);
 void gtkut_container_remove		(GtkContainer	*container,
 					 GtkWidget	*widget);
 
-gboolean gtkut_stext_match_string	(GtkSText	*text,
-					 gint		 pos,
-					 wchar_t	*wcs,
-					 gint		 len,
-					 gboolean	 case_sens);
-guint gtkut_stext_str_compare_n		(GtkSText	*text,
-					 guint		 pos1,
-					 guint		 pos2,
-					 guint		 len,
-					 guint		 text_len);
-guint gtkut_stext_str_compare		(GtkSText	*text,
-					 guint		 start_pos,
-					 guint		 text_len,
-					 const gchar	*str);
-gint gtkut_stext_find			(GtkSText	*text,
-					 guint		 start_pos,
-					 const gchar	*str,
-					 gboolean	 case_sens);
-gboolean gtkut_stext_is_uri_string	(GtkSText	*text,
-					 guint		 start_pos,
-					 guint		 text_len);
-void gtkut_stext_clear			(GtkSText	*text);
+void gtkut_scrolled_window_reset_position
+					(GtkScrolledWindow	*window);
+
+/* TextView functions */
+
+gboolean gtkut_text_buffer_match_string	(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 gunichar		*wcs,
+					 gint			 len,
+					 gboolean		 case_sens);
+gboolean gtkut_text_buffer_find		(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 const gchar		*str,
+					 gboolean		 case_sens,
+					 GtkTextIter		*match_pos);
+gboolean gtkut_text_buffer_find_backward(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 const gchar		*str,
+					 gboolean		 case_sens,
+					 GtkTextIter		*match_pos);
+
+void gtkut_text_buffer_insert_with_tag_by_name
+					(GtkTextBuffer		*buffer,
+					 GtkTextIter		*iter,
+					 const gchar		*text,
+					 gint			 len,
+					 const gchar		*tag);
+
+gchar *gtkut_text_view_get_selection	(GtkTextView	*textview);
 
 void gtkut_window_popup			(GtkWidget	*window);
+gboolean gtkut_window_modal_exist	(void);
 
 void gtkut_widget_get_uposition		(GtkWidget	*widget,
 					 gint		*px,
 					 gint		*py);
-void gtkut_widget_disable_theme_engine	(GtkWidget	*widget);
-void gtkut_widget_wait_for_draw		(GtkWidget	*widget);
+void gtkut_widget_draw_now		(GtkWidget	*widget);
 void gtkut_widget_init			(void);
 
+void gtkut_events_flush			(void);
+
 #endif /* __GTKUTILS_H__ */
