@@ -54,8 +54,8 @@
 #include "manage_window.h"
 #include "stock_pixmap.h"
 #include "progressdialog.h"
-#include "inputdialog.h"
 #include "alertpanel.h"
+#include "trayicon.h"
 #include "filter.h"
 #include "folder.h"
 
@@ -71,6 +71,9 @@ static GdkPixbuf *ok_pixbuf;
 
 static void inc_finished		(MainWindow		*mainwin,
 					 gint			 new_messages);
+
+static gint inc_remote_account_mail	(MainWindow		*mainwin,
+					 PrefsAccount		*account);
 static gint inc_account_mail_real	(MainWindow		*mainwin,
 					 PrefsAccount		*account);
 
@@ -149,7 +152,17 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	debug_print("inc_finished(): %d new message(s)\n", new_messages);
 
 	if (prefs_common.scan_all_after_inc)
-		folderview_check_new(NULL);
+		new_messages += folderview_check_new(NULL);
+
+	if (new_messages > 0) {
+		gchar buf[1024];
+
+		g_snprintf(buf, sizeof(buf), _("Sylpheed: %d new messages"),
+			   new_messages);
+		trayicon_set_tooltip(buf);
+	} else {
+		trayicon_set_tooltip(NULL);
+	}
 
 	if (new_messages <= 0 && !prefs_common.scan_all_after_inc) return;
 
@@ -169,10 +182,9 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	    prefs_common.enable_newmsg_notify &&
 	    prefs_common.newmsg_notify_cmd) {
 		gchar buf[1024];
-		gchar *p;
 
-		if ((p = strchr(prefs_common.newmsg_notify_cmd, '%')) &&
-		    *(p + 1) == 'd' && !strchr(p + 2, '%'))
+		if (str_find_format_times
+			(prefs_common.newmsg_notify_cmd, 'd') == 1)
 			g_snprintf(buf, sizeof(buf),
 				   prefs_common.newmsg_notify_cmd,
 				   new_messages);
@@ -221,20 +233,37 @@ void inc_mail(MainWindow *mainwin)
 	inc_autocheck_timer_set();
 }
 
+static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
+{
+	FolderItem *item = mainwin->summaryview->folder_item;
+	gint new_msgs;
+
+	g_return_val_if_fail(account->folder != NULL, 0);
+
+	if (account->protocol == A_IMAP4 && account->imap_check_inbox_only) {
+		FolderItem *inbox = FOLDER(account->folder)->inbox;
+
+		new_msgs = folderview_check_new_item(inbox);
+		if (!prefs_common.scan_all_after_inc && item != NULL &&
+		    inbox == item)
+			folderview_update_item(item, TRUE);
+	} else {
+		new_msgs = folderview_check_new(FOLDER(account->folder));
+		if (!prefs_common.scan_all_after_inc && item != NULL &&
+		    FOLDER(account->folder) == item->folder)
+			folderview_update_item(item, TRUE);
+	}
+
+	return new_msgs;
+}
+
 static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
 
-	if (account->protocol == A_IMAP4 || account->protocol == A_NNTP) {
-		FolderItem *item = mainwin->summaryview->folder_item;
-
-		folderview_check_new(FOLDER(account->folder));
-		if (!prefs_common.scan_all_after_inc && item != NULL &&
-		    FOLDER(account->folder) == item->folder)
-			folderview_update_item(item, TRUE);
-		return 1;
-	}
+	if (account->protocol == A_IMAP4 || account->protocol == A_NNTP)
+		return inc_remote_account_mail(mainwin, account);
 
 	session = inc_session_new(account);
 	if (!session) return 0;
@@ -297,14 +326,8 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 	for (list = account_get_list(); list != NULL; list = list->next) {
 		PrefsAccount *account = list->data;
 		if ((account->protocol == A_IMAP4 ||
-		     account->protocol == A_NNTP) && account->recv_at_getall) {
-			FolderItem *item = mainwin->summaryview->folder_item;
-
-			folderview_check_new(FOLDER(account->folder));
-			if (!prefs_common.scan_all_after_inc && item != NULL &&
-			    FOLDER(account->folder) == item->folder)
-				folderview_update_item(item, TRUE);
-		}
+		     account->protocol == A_NNTP) && account->recv_at_getall)
+			new_msgs += inc_remote_account_mail(mainwin, account);
 	}
 
 	/* check POP3 accounts */
@@ -485,7 +508,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 					(inc_dialog->dialog->window,
 					 NULL, NULL);
 
-			pass = input_dialog_query_password
+			pass = input_query_password
 				(pop3_session->ac_prefs->recv_server,
 				 pop3_session->user);
 
@@ -819,8 +842,8 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	gchar buf[MSGBUFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
 	gchar *total_size_str;
-	gint cur_total;
-	gint total;
+	gint64 cur_total;
+	gint64 total;
 
 	if (!pop3_session->new_msg_exist) return;
 
@@ -931,7 +954,7 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 	IncSession *inc_session = (IncSession *)data;
 	Pop3Session *pop3_session = POP3_SESSION(session);
 	IncProgressDialog *inc_dialog;
-	gint cur_total;
+	gint64 cur_total;
 
 	g_return_val_if_fail(inc_session != NULL, -1);
 
@@ -1053,13 +1076,9 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 
 		val = GPOINTER_TO_INT(g_hash_table_lookup
 				      (inc_session->folder_table, drop_folder));
-		if (val == 0) {
-			/* force updating */
-			if (FOLDER_IS_LOCAL(drop_folder->folder))
-				drop_folder->mtime = 0;
-			g_hash_table_insert(inc_session->folder_table, drop_folder,
-					    GINT_TO_POINTER(1));
-		}
+		if (val == 0)
+			g_hash_table_insert(inc_session->folder_table,
+					    drop_folder, GINT_TO_POINTER(1));
 		g_hash_table_insert(inc_session->tmp_folder_table, drop_folder,
 				    GINT_TO_POINTER(1));
 	}
