@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 
 typedef struct _MainWindow	MainWindow;
 
+#include "enums.h"
 #include "folderview.h"
 #include "summaryview.h"
 #include "headerview.h"
@@ -42,14 +43,6 @@ typedef enum
 	SEPARATE_BOTH	 = (SEPARATE_FOLDER | SEPARATE_MESSAGE)
 } SeparateType;
 
-typedef enum
-{
-	TOOLBAR_NONE	= 0,
-	TOOLBAR_ICON	= 1,
-	TOOLBAR_TEXT	= 2,
-	TOOLBAR_BOTH	= 3
-} ToolbarStyle;
-
 struct _MainWindow
 {
 	SeparateType type;
@@ -84,19 +77,17 @@ struct _MainWindow
 	GtkWidget *toolbar;
 	GtkWidget *get_btn;
 	GtkWidget *getall_btn;
+	GtkWidget *send_btn;
 	GtkWidget *compose_btn;
 	GtkWidget *reply_btn;
 	ComboButton *reply_combo;
 	GtkWidget *replyall_btn;
 	GtkWidget *fwd_btn;
 	ComboButton *fwd_combo;
-	GtkWidget *send_btn;
-	GtkWidget *prefs_btn;
-	GtkWidget *account_btn;
-	GtkWidget *next_btn;
 	GtkWidget *delete_btn;
 	GtkWidget *junk_btn;
 	GtkWidget *exec_btn;
+	GtkWidget *next_btn;
 
 	/* body */
 	GtkWidget *vbox_body;
@@ -111,6 +102,8 @@ struct _MainWindow
 	GtkWidget *ac_label;
 	GtkWidget *ac_menu;
 
+	GtkWidget *tray_icon;
+
 	/* context IDs for status bar */
 	gint mainwin_cid;
 	gint folderview_cid;
@@ -123,6 +116,9 @@ struct _MainWindow
 	guint menu_lock_count;
 	guint cursor_count;
 
+	gboolean window_hidden;
+	gboolean window_obscured;
+
 	FolderView	*folderview;
 	SummaryView	*summaryview;
 	MessageView	*messageview;
@@ -160,6 +156,9 @@ void main_window_progress_set		(MainWindow	*mainwin,
 					 gint		 cur,
 					 gint		 total);
 
+void main_window_progress_show		(gint		 cur,
+					 gint		 total);
+
 void main_window_toggle_online			(MainWindow	*mainwin,
 						 gboolean	 online);
 gboolean main_window_toggle_online_if_offline	(MainWindow	*mainwin);
@@ -167,6 +166,7 @@ gboolean main_window_toggle_online_if_offline	(MainWindow	*mainwin);
 void main_window_empty_trash		(MainWindow	*mainwin,
 					 gboolean	 confirm);
 void main_window_add_mailbox		(MainWindow	*mainwin);
+void main_window_send_queue		(MainWindow	*mainwin);
 
 void main_window_set_toolbar_sensitive	(MainWindow	*mainwin);
 void main_window_set_menu_sensitive	(MainWindow	*mainwin);
