@@ -22,14 +22,15 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkwindow.h>
 
-#include "intl.h"
 #include "menu.h"
 #include "utils.h"
 
@@ -44,7 +45,7 @@ GtkWidget *menubar_create(GtkWidget *window, GtkItemFactoryEntry *entries,
 	gtk_item_factory_set_translate_func(factory, menu_translate,
 					    NULL, NULL);
 	gtk_item_factory_create_items(factory, n_entries, entries, data);
-	gtk_accel_group_attach(factory->accel_group, GTK_OBJECT(window));
+	gtk_window_add_accel_group(GTK_WINDOW(window), factory->accel_group);
 
 	return gtk_item_factory_get_widget(factory, path);
 }
@@ -70,6 +71,7 @@ static gchar *menu_translate(const gchar *path, gpointer data)
 	return retval;
 }
 
+#if 0
 static void factory_print_func(gpointer data, gchar *string)
 {
 	GString *out_str = data;
@@ -150,6 +152,7 @@ void menu_factory_copy_rc(const gchar *src_path, const gchar *dest_path)
 	gtk_item_factory_parse_rc_string(string->str);
 	g_string_free(string, TRUE);
 }
+#endif
 
 void menu_set_sensitive(GtkItemFactory *ifactory, const gchar *path,
 			gboolean sensitive)
@@ -181,7 +184,8 @@ void menu_set_active(GtkItemFactory *ifactory, const gchar *path,
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), is_active);
 }
 
-void menu_button_position(GtkMenu *menu, gint *x, gint *y, gpointer user_data)
+void menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
+			  gpointer user_data)
 {
 	GtkWidget *button;
 	GtkRequisition requisition;
@@ -190,7 +194,7 @@ void menu_button_position(GtkMenu *menu, gint *x, gint *y, gpointer user_data)
 	gint width, height;
 	gint scr_width, scr_height;
 
-	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(x != NULL && y != NULL);
 	g_return_if_fail(GTK_IS_BUTTON(user_data));
 
 	button = GTK_WIDGET(user_data);
@@ -200,8 +204,8 @@ void menu_button_position(GtkMenu *menu, gint *x, gint *y, gpointer user_data)
 	height = requisition.height;
 	gdk_window_get_origin(button->window, &button_xpos, &button_ypos);
 
-	xpos = button_xpos;
-	ypos = button_ypos + button->allocation.height;
+	xpos = button_xpos + button->allocation.x;
+	ypos = button_ypos + button->allocation.y + button->requisition.height;
 
 	scr_width = gdk_screen_width();
 	scr_height = gdk_screen_height();
@@ -209,7 +213,7 @@ void menu_button_position(GtkMenu *menu, gint *x, gint *y, gpointer user_data)
 	if (xpos + width > scr_width)
 		xpos -= (xpos + width) - scr_width;
 	if (ypos + height > scr_height)
-		ypos = button_ypos - height;
+		ypos -= button->requisition.height + height;
 	if (xpos < 0)
 		xpos = 0;
 	if (ypos < 0)
@@ -233,7 +237,7 @@ gint menu_find_option_menu_index(GtkOptionMenu *optmenu, gpointer data,
 	for (cur = GTK_MENU_SHELL(menu)->children, n = 0;
 	     cur != NULL; cur = cur->next, n++) {
 		menuitem = GTK_WIDGET(cur->data);
-		menu_data = gtk_object_get_user_data(GTK_OBJECT(menuitem));
+		menu_data = g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID);
 		if (func) {
 			if (func(menu_data, data) == 0)
 				return n;
@@ -244,7 +248,7 @@ gint menu_find_option_menu_index(GtkOptionMenu *optmenu, gpointer data,
 	return -1;
 }
 
-gpointer menu_get_option_menu_active_user_data(GtkOptionMenu *optmenu)
+gint menu_get_option_menu_active_index(GtkOptionMenu *optmenu)
 {
 	GtkWidget *menu;
 	GtkWidget *menuitem;
@@ -252,5 +256,6 @@ gpointer menu_get_option_menu_active_user_data(GtkOptionMenu *optmenu)
 	menu = gtk_option_menu_get_menu(optmenu);
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 
-	return gtk_object_get_user_data(GTK_OBJECT(menuitem));
+	return GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
