@@ -28,18 +28,6 @@
 
 #include "procmsg.h"
 
-typedef enum
-{
-	FILTER_BY_NONE,
-	FILTER_BY_AUTO,
-	FILTER_BY_FROM,
-	FILTER_BY_TO,
-	FILTER_BY_SUBJECT
-} PrefsFilterType;
-
-void prefs_filter_read_config	(void);
-void prefs_filter_write_config	(void);
-
 void prefs_filter_open		(MsgInfo	*msginfo,
 				 const gchar	*header);
 
@@ -48,11 +36,8 @@ GSList *prefs_filter_get_user_header_list	(void);
 
 gchar *prefs_filter_get_msg_header_field	(const gchar	*header_name);
 
+void prefs_filter_set_header_list		(MsgInfo	*msginfo);
 void prefs_filter_set_user_header_list		(GSList		*list);
 void prefs_filter_set_msg_header_list		(MsgInfo	*msginfo);
 
-void prefs_filter_rename_path	(const gchar	*old_path,
-				 const gchar	*new_path);
-void prefs_filter_delete_path	(const gchar	*path);
-
 #endif /* __PREFS_FILTER_H__ */
