@@ -35,6 +35,7 @@
 #include <gtk/gtkstock.h>
 
 #include "prefs.h"
+#include "prefs_ui.h"
 #include "prefs_common.h"
 #include "prefs_summary_column.h"
 #include "manage_window.h"
@@ -132,6 +133,7 @@ void prefs_summary_column_open(void)
 		gtk_main_iteration();
 
 	gtk_widget_hide(summary_col.window);
+	main_window_popup(main_window_get());
 
 	inc_unlock();
 }
