@@ -5,7 +5,8 @@
 #define quote_fmt_parse	quote_fmtparse
 
 gchar *quote_fmt_get_buffer(void);
-void quote_fmt_init(MsgInfo *info, gchar *my_quote_str);
+void quote_fmt_init(MsgInfo *info, const gchar *my_quote_str,
+		    const gchar *my_body);
 gint quote_fmtparse(void);
 void quote_fmt_scan_string(const gchar *str);
 
