@@ -44,9 +44,11 @@
 #include "prefs_account.h"
 #include "procheader.h"
 #include "account.h"
+#include "folder.h"
+#include "procmsg.h"
+#include "filter.h"
 #include "progressdialog.h"
 #include "statusbar.h"
-#include "inputdialog.h"
 #include "alertpanel.h"
 #include "manage_window.h"
 #include "socket.h"
@@ -304,6 +306,87 @@ gint send_message_queue(QueueInfo *qinfo)
 	return val;
 }
 
+gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
+			    gboolean filter_msgs)
+{
+	gint ret = 0;
+	GSList *mlist = NULL;
+	GSList *cur;
+
+	if (!queue)
+		queue = folder_get_default_queue();
+	g_return_val_if_fail(queue != NULL, -1);
+
+	mlist = folder_item_get_msg_list(queue, FALSE);
+	mlist = procmsg_sort_msg_list(mlist, SORT_BY_NUMBER, SORT_ASCENDING);
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		gchar *file;
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		QueueInfo *qinfo;
+		gchar tmp[MAXPATHLEN + 1];
+
+		file = procmsg_get_message_file(msginfo);
+		if (!file)
+			continue;
+
+		qinfo = send_get_queue_info(file);
+		if (!qinfo || send_message_queue(qinfo) < 0) {
+			g_warning("Sending queued message %d failed.\n",
+				  msginfo->msgnum);
+			send_queue_info_free(qinfo);
+			g_free(file);
+			continue;
+		}
+
+		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
+			   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
+
+		if (send_get_queue_contents(qinfo, tmp) == 0) {
+			if (save_msgs) {
+				FolderItem *outbox;
+				gboolean drop_done = FALSE;
+
+				if (filter_msgs) {
+					FilterInfo *fltinfo;
+
+					fltinfo = filter_info_new();
+					fltinfo->account = qinfo->ac;
+					fltinfo->flags.perm_flags = 0;
+					fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+					filter_apply(prefs_common.fltlist, tmp,
+						     fltinfo);
+
+					drop_done = fltinfo->drop_done;
+					filter_info_free(fltinfo);
+				}
+
+				if (!drop_done) {
+					outbox = account_get_special_folder
+						(qinfo->ac, F_OUTBOX);
+					procmsg_save_to_outbox(outbox, tmp);
+				}
+			}
+			g_unlink(tmp);
+		}
+
+		send_queue_info_free(qinfo);
+		g_free(file);
+
+		folder_item_remove_msg(queue, msginfo);
+		ret++;
+	}
+
+	procmsg_msg_list_free(mlist);
+
+	procmsg_clear_cache(queue);
+	queue->cache_dirty = FALSE;
+	queue->mtime = 0;
+
+	return ret;
+}
+
 static gint send_message_local(const gchar *command, FILE *fp)
 {
 	gchar **argv;
@@ -374,6 +457,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 {
 	Session *session;
 	SMTPSession *smtp_session;
+	FILE *out_fp;
 	gushort port;
 	SendProgressDialog *dialog;
 	gchar buf[BUFFSIZE];
@@ -404,7 +488,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 					g_strdup(ac_prefs->tmp_smtp_pass);
 			else {
 				smtp_session->pass =
-					input_dialog_query_password
+					input_query_password
 						(ac_prefs->smtp_server,
 						 smtp_session->user);
 				if (!smtp_session->pass)
@@ -421,7 +505,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 					g_strdup(ac_prefs->tmp_pass);
 			else {
 				smtp_session->pass =
-					input_dialog_query_password
+					input_query_password
 						(ac_prefs->smtp_server,
 						 smtp_session->user);
 				if (!smtp_session->pass)
@@ -438,8 +522,18 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 	smtp_session->from = g_strdup(ac_prefs->address);
 	smtp_session->to_list = to_list;
 	smtp_session->cur_to = to_list;
-	smtp_session->send_data = get_outgoing_rfc2822_str(fp);
-	smtp_session->send_data_len = strlen(smtp_session->send_data);
+
+	out_fp = get_outgoing_rfc2822_file(fp);
+	if (!out_fp) {
+		session_destroy(session);
+		return -1;
+	}
+	smtp_session->send_data_fp = out_fp;
+	smtp_session->send_data_len = get_left_file_size(out_fp);
+	if (smtp_session->send_data_len < 0) {
+		session_destroy(session);
+		return -1;
+	}
 
 #if USE_SSL
 	port = ac_prefs->set_smtpport ? ac_prefs->smtpport :
@@ -467,6 +561,8 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		(session, send_send_data_progressive, dialog);
 	session_set_send_data_notify(session, send_send_data_finished, dialog);
 
+	session_set_timeout(session, prefs_common.io_timeout_secs * 1000);
+
 	if (session_connect(session, ac_prefs->smtp_server, port) < 0) {
 		session_destroy(session);
 		send_progress_dialog_destroy(dialog);
